@@ -1,22 +1,22 @@
-/* Copyright (C) 2002 Jean-Marc Valin 
+/* Copyright (C) 2002 Jean-Marc Valin
    File: quant_lsp.c
    LSP vector quantization
 
    Redistribution and use in source and binary forms, with or without
    modification, are permitted provided that the following conditions
    are met:
-   
+
    - Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.
-   
+
    - Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.
-   
+
    - Neither the name of the Xiph.org Foundation nor the names of its
    contributors may be used to endorse or promote products derived from
    this software without specific prior written permission.
-   
+
    THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
    ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
    LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
@@ -35,21 +35,25 @@
 #endif
 
 #include "quant_lsp.h"
+#include "os_support.h"
 #include <math.h>
 #ifndef M_PI
 #define M_PI 3.14159265358979323846
 #endif
 
+#include "arch.h"
 
-#include "misc.h"
+#ifdef BFIN_ASM
+#include "quant_lsp_bfin.h"
+#endif
 
 #ifdef FIXED_POINT
 
-#define LSP_LINEAR(i) (SHL(i+1,11))
+#define LSP_LINEAR(i) (SHL16(i+1,11))
 #define LSP_LINEAR_HIGH(i) (ADD16(MULT16_16_16(i,2560),6144))
-#define LSP_DIV_256(x) (SHL((spx_word16_t)x, 5))
-#define LSP_DIV_512(x) (SHL((spx_word16_t)x, 4))
-#define LSP_DIV_1024(x) (SHL((spx_word16_t)x, 3))
+#define LSP_DIV_256(x) (SHL16((spx_word16_t)x, 5))
+#define LSP_DIV_512(x) (SHL16((spx_word16_t)x, 4))
+#define LSP_DIV_1024(x) (SHL16((spx_word16_t)x, 3))
 #define LSP_PI 25736
 
 #else
@@ -64,6 +68,7 @@
 
 #endif
 
+#ifndef DISABLE_ENCODER
 static void compute_quant_weights(spx_lsp_t *qlsp, spx_word16_t *quant_weight, int order)
 {
    int i;
@@ -90,12 +95,13 @@ static void compute_quant_weights(spx_lsp_t *qlsp, spx_word16_t *quant_weight, i
 }
 
 /* Note: x is modified*/
+#ifndef OVERRIDE_LSP_QUANT
 static int lsp_quant(spx_word16_t *x, const signed char *cdbk, int nbVec, int nbDim)
 {
    int i,j;
    spx_word32_t dist;
    spx_word16_t tmp;
-   spx_word32_t best_dist=0;
+   spx_word32_t best_dist=VERY_LARGE32;
    int best_id=0;
    const signed char *ptr=cdbk;
    for (i=0;i<nbVec;i++)
@@ -103,10 +109,10 @@ static int lsp_quant(spx_word16_t *x, const signed char *cdbk, int nbVec, int nb
       dist=0;
       for (j=0;j<nbDim;j++)
       {
-         tmp=SUB16(x[j],SHL((spx_word16_t)*ptr++,5));
+         tmp=SUB16(x[j],SHL16((spx_word16_t)*ptr++,5));
          dist=MAC16_16(dist,tmp,tmp);
       }
-      if (dist<best_dist || i==0)
+      if (dist<best_dist)
       {
          best_dist=dist;
          best_id=i;
@@ -114,18 +120,20 @@ static int lsp_quant(spx_word16_t *x, const signed char *cdbk, int nbVec, int nb
    }
 
    for (j=0;j<nbDim;j++)
-      x[j] -= SHL((spx_word16_t)cdbk[best_id*nbDim+j],5);
-    
+      x[j] = SUB16(x[j],SHL16((spx_word16_t)cdbk[best_id*nbDim+j],5));
+
    return best_id;
 }
+#endif
 
 /* Note: x is modified*/
+#ifndef OVERRIDE_LSP_WEIGHT_QUANT
 static int lsp_weight_quant(spx_word16_t *x, spx_word16_t *weight, const signed char *cdbk, int nbVec, int nbDim)
 {
    int i,j;
    spx_word32_t dist;
    spx_word16_t tmp;
-   spx_word32_t best_dist=0;
+   spx_word32_t best_dist=VERY_LARGE32;
    int best_id=0;
    const signed char *ptr=cdbk;
    for (i=0;i<nbVec;i++)
@@ -133,35 +141,35 @@ static int lsp_weight_quant(spx_word16_t *x, spx_word16_t *weight, const signed
       dist=0;
       for (j=0;j<nbDim;j++)
       {
-         tmp=SUB16(x[j],SHL((spx_word16_t)*ptr++,5));
+         tmp=SUB16(x[j],SHL16((spx_word16_t)*ptr++,5));
          dist=MAC16_32_Q15(dist,weight[j],MULT16_16(tmp,tmp));
       }
-      if (dist<best_dist || i==0)
+      if (dist<best_dist)
       {
          best_dist=dist;
          best_id=i;
       }
    }
-   
+
    for (j=0;j<nbDim;j++)
-      x[j] -= SHL((spx_word16_t)cdbk[best_id*nbDim+j],5);
+      x[j] = SUB16(x[j],SHL16((spx_word16_t)cdbk[best_id*nbDim+j],5));
    return best_id;
 }
-
+#endif
 
 void lsp_quant_nb(spx_lsp_t *lsp, spx_lsp_t *qlsp, int order, SpeexBits *bits)
 {
    int i;
    int id;
    spx_word16_t quant_weight[10];
-   
+
    for (i=0;i<order;i++)
       qlsp[i]=lsp[i];
 
    compute_quant_weights(qlsp, quant_weight, order);
 
    for (i=0;i<order;i++)
-      qlsp[i]-=LSP_LINEAR(i);
+      qlsp[i]=SUB16(qlsp[i],LSP_LINEAR(i));
 
 #ifndef FIXED_POINT
    for (i=0;i<order;i++)
@@ -172,7 +180,7 @@ void lsp_quant_nb(spx_lsp_t *lsp, spx_lsp_t *qlsp, int order, SpeexBits *bits)
 
    for (i=0;i<order;i++)
       qlsp[i]*=2;
- 
+
    id = lsp_weight_quant(qlsp, quant_weight, cdbk_nb_low1, NB_CDBK_SIZE_LOW1, 5);
    speex_bits_pack(bits, id, 6);
 
@@ -193,7 +201,7 @@ void lsp_quant_nb(spx_lsp_t *lsp, spx_lsp_t *qlsp, int order, SpeexBits *bits)
 
 #ifdef FIXED_POINT
    for (i=0;i<order;i++)
-      qlsp[i]=PSHR(qlsp[i],2);
+      qlsp[i]=PSHR16(qlsp[i],2);
 #else
    for (i=0;i<order;i++)
       qlsp[i]=qlsp[i] * .00097656;
@@ -202,7 +210,9 @@ void lsp_quant_nb(spx_lsp_t *lsp, spx_lsp_t *qlsp, int order, SpeexBits *bits)
    for (i=0;i<order;i++)
       qlsp[i]=lsp[i]-qlsp[i];
 }
+#endif /* DISABLE_ENCODER */
 
+#ifndef DISABLE_DECODER
 void lsp_unquant_nb(spx_lsp_t *lsp, int order, SpeexBits *bits)
 {
    int i, id;
@@ -212,26 +222,27 @@ void lsp_unquant_nb(spx_lsp_t *lsp, int order, SpeexBits *bits)
 
    id=speex_bits_unpack_unsigned(bits, 6);
    for (i=0;i<10;i++)
-      lsp[i] += LSP_DIV_256(cdbk_nb[id*10+i]);
+      lsp[i] = ADD32(lsp[i], LSP_DIV_256(cdbk_nb[id*10+i]));
 
    id=speex_bits_unpack_unsigned(bits, 6);
    for (i=0;i<5;i++)
-      lsp[i] += LSP_DIV_512(cdbk_nb_low1[id*5+i]);
+      lsp[i] = ADD16(lsp[i], LSP_DIV_512(cdbk_nb_low1[id*5+i]));
 
    id=speex_bits_unpack_unsigned(bits, 6);
    for (i=0;i<5;i++)
-      lsp[i] += LSP_DIV_1024(cdbk_nb_low2[id*5+i]);
+      lsp[i] = ADD32(lsp[i], LSP_DIV_1024(cdbk_nb_low2[id*5+i]));
 
    id=speex_bits_unpack_unsigned(bits, 6);
    for (i=0;i<5;i++)
-      lsp[i+5] += LSP_DIV_512(cdbk_nb_high1[id*5+i]);
-   
+      lsp[i+5] = ADD32(lsp[i+5], LSP_DIV_512(cdbk_nb_high1[id*5+i]));
+
    id=speex_bits_unpack_unsigned(bits, 6);
    for (i=0;i<5;i++)
-      lsp[i+5] += LSP_DIV_1024(cdbk_nb_high2[id*5+i]);
+      lsp[i+5] = ADD32(lsp[i+5], LSP_DIV_1024(cdbk_nb_high2[id*5+i]));
 }
+#endif /* DISABLE_DECODER */
 
-
+#ifndef DISABLE_ENCODER
 void lsp_quant_lbr(spx_lsp_t *lsp, spx_lsp_t *qlsp, int order, SpeexBits *bits)
 {
    int i;
@@ -244,17 +255,17 @@ void lsp_quant_lbr(spx_lsp_t *lsp, spx_lsp_t *qlsp, int order, SpeexBits *bits)
    compute_quant_weights(qlsp, quant_weight, order);
 
    for (i=0;i<order;i++)
-      qlsp[i]-=LSP_LINEAR(i);
+      qlsp[i]=SUB16(qlsp[i],LSP_LINEAR(i));
 #ifndef FIXED_POINT
    for (i=0;i<order;i++)
       qlsp[i]=qlsp[i]*LSP_SCALE;
 #endif
    id = lsp_quant(qlsp, cdbk_nb, NB_CDBK_SIZE, order);
    speex_bits_pack(bits, id, 6);
-   
+
    for (i=0;i<order;i++)
       qlsp[i]*=2;
-   
+
    id = lsp_weight_quant(qlsp, quant_weight, cdbk_nb_low1, NB_CDBK_SIZE_LOW1, 5);
    speex_bits_pack(bits, id, 6);
 
@@ -263,7 +274,7 @@ void lsp_quant_lbr(spx_lsp_t *lsp, spx_lsp_t *qlsp, int order, SpeexBits *bits)
 
 #ifdef FIXED_POINT
    for (i=0;i<order;i++)
-      qlsp[i] = PSHR(qlsp[i],1);
+      qlsp[i] = PSHR16(qlsp[i],1);
 #else
    for (i=0;i<order;i++)
       qlsp[i] = qlsp[i]*0.0019531;
@@ -272,7 +283,9 @@ void lsp_quant_lbr(spx_lsp_t *lsp, spx_lsp_t *qlsp, int order, SpeexBits *bits)
    for (i=0;i<order;i++)
       qlsp[i]=lsp[i]-qlsp[i];
 }
+#endif /* DISABLE_ENCODER */
 
+#ifndef DISABLE_DECODER
 void lsp_unquant_lbr(spx_lsp_t *lsp, int order, SpeexBits *bits)
 {
    int i, id;
@@ -291,14 +304,15 @@ void lsp_unquant_lbr(spx_lsp_t *lsp, int order, SpeexBits *bits)
    id=speex_bits_unpack_unsigned(bits, 6);
    for (i=0;i<5;i++)
       lsp[i+5] += LSP_DIV_512(cdbk_nb_high1[id*5+i]);
-   
+
 }
+#endif /* DISABLE_DECODER */
 
-
+#ifndef DISABLE_WIDEBAND
 extern const signed char high_lsp_cdbk[];
 extern const signed char high_lsp_cdbk2[];
 
-
+#ifndef DISABLE_ENCODER
 void lsp_quant_high(spx_lsp_t *lsp, spx_lsp_t *qlsp, int order, SpeexBits *bits)
 {
    int i;
@@ -320,7 +334,7 @@ void lsp_quant_high(spx_lsp_t *lsp, spx_lsp_t *qlsp, int order, SpeexBits *bits)
       }*/
 
    for (i=0;i<order;i++)
-      qlsp[i]-=LSP_LINEAR_HIGH(i);
+      qlsp[i]=SUB16(qlsp[i],LSP_LINEAR_HIGH(i));
 #ifndef FIXED_POINT
    for (i=0;i<order;i++)
       qlsp[i] = qlsp[i]*LSP_SCALE;
@@ -336,7 +350,7 @@ void lsp_quant_high(spx_lsp_t *lsp, spx_lsp_t *qlsp, int order, SpeexBits *bits)
 
 #ifdef FIXED_POINT
    for (i=0;i<order;i++)
-      qlsp[i] = PSHR(qlsp[i],1);
+      qlsp[i] = PSHR16(qlsp[i],1);
 #else
    for (i=0;i<order;i++)
       qlsp[i] = qlsp[i]*0.0019531;
@@ -345,7 +359,10 @@ void lsp_quant_high(spx_lsp_t *lsp, spx_lsp_t *qlsp, int order, SpeexBits *bits)
    for (i=0;i<order;i++)
       qlsp[i]=lsp[i]-qlsp[i];
 }
+#endif /* DISABLE_ENCODER */
 
+
+#ifndef DISABLE_DECODER
 void lsp_unquant_high(spx_lsp_t *lsp, int order, SpeexBits *bits)
 {
 
@@ -363,67 +380,7 @@ void lsp_unquant_high(spx_lsp_t *lsp, int order, SpeexBits *bits)
    for (i=0;i<order;i++)
       lsp[i] += LSP_DIV_512(high_lsp_cdbk2[id*order+i]);
 }
+#endif /* DISABLE_DECODER */
 
+#endif /* DISABLE_WIDEBAND */
 
-#ifdef EPIC_48K
-
-extern const signed char cdbk_lsp_vlbr[5120];
-extern const signed char cdbk_lsp2_vlbr[160];
-
-void lsp_quant_48k(spx_lsp_t *lsp, spx_lsp_t *qlsp, int order, SpeexBits *bits)
-{
-   int i;
-   int id;
-   spx_word16_t quant_weight[10];
-
-   for (i=0;i<order;i++)
-      qlsp[i]=lsp[i];
-
-   compute_quant_weights(qlsp, quant_weight, order);
-
-   for (i=0;i<order;i++)
-      qlsp[i]-=LSP_SCALING*(.25*i+.3125);
-#ifndef FIXED_POINT
-   for (i=0;i<order;i++)
-      qlsp[i] = qlsp[i]*LSP_SCALE;
-#endif
-   
-   id = lsp_quant(qlsp, cdbk_lsp_vlbr, 512, order);
-   speex_bits_pack(bits, id, 9);
-
-   for (i=0;i<order;i++)
-      qlsp[i]*=4;
-   
-   id = lsp_weight_quant(qlsp, quant_weight, cdbk_lsp2_vlbr, 16, 10);
-   speex_bits_pack(bits, id, 4);
-
-#ifdef FIXED_POINT
-   for (i=0;i<order;i++)
-      qlsp[i]=PSHR(qlsp[i],2);
-#else
-   for (i=0;i<order;i++)
-      qlsp[i]=qlsp[i]*0.00097655;
-#endif
-
-   for (i=0;i<order;i++)
-      qlsp[i]=lsp[i]-qlsp[i];
-}
-
-void lsp_unquant_48k(spx_lsp_t *lsp, int order, SpeexBits *bits)
-{
-   int i, id;
-   for (i=0;i<order;i++)
-      lsp[i]=LSP_SCALING*(.25*i+.3125);
-
-
-   id=speex_bits_unpack_unsigned(bits, 9);
-   for (i=0;i<10;i++)
-      lsp[i] += LSP_SCALING*0.0039062*cdbk_lsp_vlbr[id*10+i];
-
-   id=speex_bits_unpack_unsigned(bits, 4);
-   for (i=0;i<10;i++)
-      lsp[i] += LSP_SCALING*0.00097655*cdbk_lsp2_vlbr[id*10+i];
-   
-}
-
-#endif
