@@ -1,21 +1,21 @@
-/* Copyright (C) 2002-2006 Jean-Marc Valin 
+/* Copyright (C) 2002-2006 Jean-Marc Valin
    File: speexdec.c
 
    Redistribution and use in source and binary forms, with or without
    modification, are permitted provided that the following conditions
    are met:
-   
+
    - Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.
-   
+
    - Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.
-   
+
    - Neither the name of the Xiph.org Foundation nor the names of its
    contributors may be used to endorse or promote products derived from
    this software without specific prior written permission.
-   
+
    THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
    ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
    LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
@@ -36,6 +36,8 @@
 #include <stdio.h>
 #if !defined WIN32 && !defined _WIN32
 #include <unistd.h>
+#endif
+#ifdef HAVE_GETOPT_H
 #include <getopt.h>
 #endif
 #ifndef HAVE_GETOPT_LONG
@@ -44,12 +46,9 @@
 #include <stdlib.h>
 #include <string.h>
 
-#include <speex/speex.h>
 #include <ogg/ogg.h>
 
 #if defined WIN32 || defined _WIN32
-#include <windows.h>
-#include "getopt_win.h"
 #include "wave_out.h"
 /* We need the following two to set stdout to binary */
 #include <io.h>
@@ -81,33 +80,32 @@
 
 #include <string.h>
 #include "wav_io.h"
-#include <speex/speex_header.h>
-#include <speex/speex_stereo.h>
-#include <speex/speex_callbacks.h>
-#include "wav_io.h"
+#include "speex/speex_header.h"
+#include "speex/speex_stereo.h"
+#include "speex/speex_callbacks.h"
 
 #define MAX_FRAME_SIZE 2000
 
-#define readint(buf, base) (((buf[base+3]<<24)&0xff000000)| \
-                           ((buf[base+2]<<16)&0xff0000)| \
-                           ((buf[base+1]<<8)&0xff00)| \
-  	           	    (buf[base]&0xff))
+#define readint(buf) ((((unsigned)buf[3]<<24)&0xff000000)| \
+                      (((unsigned)buf[2]<<16)&0xff0000)| \
+                      (((unsigned)buf[1]<<8)&0xff00)| \
+                       ((unsigned)buf[0]&0xff))
 
 static void print_comments(char *comments, int length)
 {
    char *c=comments;
    int len, i, nb_fields;
    char *end;
-   
+
    if (length<8)
    {
       fprintf (stderr, "Invalid/corrupted comments\n");
       return;
    }
    end = c+length;
-   len=readint(c, 0);
+   len=readint(c);
    c+=4;
-   if (c+len>end)
+   if (len < 0 || c+len>end)
    {
       fprintf (stderr, "Invalid/corrupted comments\n");
       return;
@@ -120,7 +118,7 @@ static void print_comments(char *comments, int length)
       fprintf (stderr, "Invalid/corrupted comments\n");
       return;
    }
-   nb_fields=readint(c, 0);
+   nb_fields=readint(c);
    c+=4;
    for (i=0;i<nb_fields;i++)
    {
@@ -129,9 +127,9 @@ static void print_comments(char *comments, int length)
          fprintf (stderr, "Invalid/corrupted comments\n");
          return;
       }
-      len=readint(c, 0);
+      len=readint(c);
       c+=4;
-      if (c+len>end)
+      if (len < 0 || c+len>end)
       {
          fprintf (stderr, "Invalid/corrupted comments\n");
          return;
@@ -154,7 +152,7 @@ FILE *out_file_open(char *outFile, int rate, int *channels)
       if (audio_fd<0)
       {
          perror("Cannot open /dev/dsp");
-         exit(1);         
+         exit(1);
       }
 
       format=AFMT_S16_NE;
@@ -191,7 +189,7 @@ FILE *out_file_open(char *outFile, int rate, int *channels)
 #elif defined HAVE_SYS_AUDIOIO_H
       audio_info_t info;
       int audio_fd;
-      
+
       audio_fd = open("/dev/audio", O_WRONLY);
       if (audio_fd<0)
       {
@@ -207,7 +205,7 @@ FILE *out_file_open(char *outFile, int rate, int *channels)
       info.play.precision = 16;
       info.play.sample_rate = rate;
       info.play.channels = *channels;
-      
+
       if (ioctl(audio_fd, AUDIO_SETINFO, &info) < 0)
       {
          perror ("AUDIO_SETINFO");
@@ -232,10 +230,12 @@ FILE *out_file_open(char *outFile, int rate, int *channels)
       {
 #if defined WIN32 || defined _WIN32
          _setmode(_fileno(stdout), _O_BINARY);
+#elif defined OS2
+         _fsetmode(stdout,"b");
 #endif
          fout=stdout;
       }
-      else 
+      else
       {
          fout = fopen(outFile, "wb");
          if (!fout)
@@ -259,13 +259,13 @@ void usage()
    printf ("input_file can be:\n");
    printf ("  filename.spx         regular Speex file\n");
    printf ("  -                    stdin\n");
-   printf ("\n");  
+   printf ("\n");
    printf ("output_file can be:\n");
    printf ("  filename.wav         Wav file\n");
    printf ("  filename.*           Raw PCM file (any extension other that .wav)\n");
    printf ("  -                    stdout\n");
    printf ("  (nothing)            Will be played to soundcard\n");
-   printf ("\n");  
+   printf ("\n");
    printf ("Options:\n");
    printf (" --enh                 Enable perceptual enhancement (default)\n");
    printf (" --no-enh              Disable perceptual enhancement\n");
@@ -276,7 +276,7 @@ void usage()
    printf (" --stereo              Force decoding in stereo\n");
    printf (" --rate n              Force decoding at sampling rate n Hz\n");
    printf (" --packet-loss n       Simulate n %% random packet loss\n");
-   printf (" -V                    Verbose mode (show bit-rate)\n"); 
+   printf (" -V                    Verbose mode (show bit-rate)\n");
    printf (" -h, --help            This help\n");
    printf (" -v, --version         Version information\n");
    printf (" --pf                  Deprecated, use --enh instead\n");
@@ -289,109 +289,128 @@ void usage()
 
 void version()
 {
-   printf ("speexdec (Speex decoder) version " SPEEX_VERSION " (compiled " __DATE__ ")\n");
+   const char* speex_version;
+   speex_lib_ctl(SPEEX_LIB_GET_VERSION_STRING, (void*)&speex_version);
+   printf ("speexdec (Speex decoder) version %s (compiled " __DATE__ ")\n", speex_version);
    printf ("Copyright (C) 2002-2006 Jean-Marc Valin\n");
 }
 
 void version_short()
 {
-   printf ("speexdec version " SPEEX_VERSION "\n");
+   const char* speex_version;
+   speex_lib_ctl(SPEEX_LIB_GET_VERSION_STRING, (void*)&speex_version);
+   printf ("speexdec version %s\n", speex_version);
    printf ("Copyright (C) 2002-2006 Jean-Marc Valin\n");
 }
 
-static void *process_header(ogg_packet *op, int enh_enabled, int *frame_size, int *rate, int *nframes, int forceMode, int *channels, SpeexStereoState *stereo, int *extra_headers, int quiet)
+static void *process_header(ogg_packet *op, spx_int32_t enh_enabled, spx_int32_t *frame_size, int *granule_frame_size, spx_int32_t *rate, int *nframes, int forceMode, int *channels, SpeexStereoState *stereo, int *extra_headers, int quiet)
 {
    void *st;
    const SpeexMode *mode;
    SpeexHeader *header;
    int modeID;
    SpeexCallback callback;
-      
+
    header = speex_packet_to_header((char*)op->packet, op->bytes);
    if (!header)
    {
       fprintf (stderr, "Cannot read header\n");
       return NULL;
    }
-   if (header->mode >= SPEEX_NB_MODES)
+   if (header->mode >= SPEEX_NB_MODES || header->mode<0)
    {
-      fprintf (stderr, "Mode number %d does not (yet/any longer) exist in this version\n", 
+      fprintf (stderr, "Mode number %d does not (yet/any longer) exist in this version\n",
                header->mode);
+      free(header);
       return NULL;
    }
-      
+
    modeID = header->mode;
    if (forceMode!=-1)
       modeID = forceMode;
 
    mode = speex_lib_get_mode (modeID);
-   
+
    if (header->speex_version_id > 1)
    {
       fprintf (stderr, "This file was encoded with Speex bit-stream version %d, which I don't know how to decode\n", header->speex_version_id);
+      free(header);
       return NULL;
    }
 
    if (mode->bitstream_version < header->mode_bitstream_version)
    {
       fprintf (stderr, "The file was encoded with a newer version of Speex. You need to upgrade in order to play it.\n");
+      free(header);
       return NULL;
    }
-   if (mode->bitstream_version > header->mode_bitstream_version) 
+   if (mode->bitstream_version > header->mode_bitstream_version)
    {
       fprintf (stderr, "The file was encoded with an older version of Speex. You would need to downgrade the version in order to play it.\n");
+      free(header);
       return NULL;
    }
-   
+
    st = speex_decoder_init(mode);
    if (!st)
    {
       fprintf (stderr, "Decoder initialization failed.\n");
+      free(header);
       return NULL;
    }
    speex_decoder_ctl(st, SPEEX_SET_ENH, &enh_enabled);
    speex_decoder_ctl(st, SPEEX_GET_FRAME_SIZE, frame_size);
+   *granule_frame_size = *frame_size;
 
-   if (!(*channels==1))
-   {
-      callback.callback_id = SPEEX_INBAND_STEREO;
-      callback.func = speex_std_stereo_request_handler;
-      callback.data = stereo;
-      speex_decoder_ctl(st, SPEEX_SET_HANDLER, &callback);
-   }
    if (!*rate)
       *rate = header->rate;
    /* Adjust rate if --force-* options are used */
    if (forceMode!=-1)
    {
       if (header->mode < forceMode)
+      {
          *rate <<= (forceMode - header->mode);
+         *granule_frame_size >>= (forceMode - header->mode);
+      }
       if (header->mode > forceMode)
+      {
          *rate >>= (header->mode - forceMode);
+         *granule_frame_size <<= (header->mode - forceMode);
+      }
    }
 
+
    speex_decoder_ctl(st, SPEEX_SET_SAMPLING_RATE, rate);
 
    *nframes = header->frames_per_packet;
 
    if (*channels==-1)
       *channels = header->nb_channels;
-   
+
+   if (!(*channels==1))
+   {
+      *channels = 2;
+      callback.callback_id = SPEEX_INBAND_STEREO;
+      callback.func = speex_std_stereo_request_handler;
+      callback.data = stereo;
+      speex_decoder_ctl(st, SPEEX_SET_HANDLER, &callback);
+   }
+
    if (!quiet)
    {
-      fprintf (stderr, "Decoding %d Hz audio using %s mode", 
+      fprintf (stderr, "Decoding %d Hz audio using %s mode",
                *rate, mode->modeName);
 
       if (*channels==1)
          fprintf (stderr, " (mono");
       else
          fprintf (stderr, " (stereo");
-      
+
       if (header->vbr)
          fprintf (stderr, ", VBR)\n");
       else
          fprintf(stderr, ")\n");
-      /*fprintf (stderr, "Decoding %d Hz audio at %d bps using %s mode\n", 
+      /*fprintf (stderr, "Decoding %d Hz audio at %d bps using %s mode\n",
        *rate, mode->bitrate, mode->modeName);*/
    }
 
@@ -409,7 +428,7 @@ int main(int argc, char **argv)
    FILE *fin, *fout=NULL;
    short out[MAX_FRAME_SIZE];
    short output[MAX_FRAME_SIZE];
-   int frame_size=0;
+   int frame_size=0, granule_frame_size=0;
    void *st=NULL;
    SpeexBits bits;
    int packet_count=0;
@@ -451,8 +470,10 @@ int main(int argc, char **argv)
    SpeexStereoState stereo = SPEEX_STEREO_STATE_INIT;
    int channels=-1;
    int rate=0;
-   int extra_headers;
+   int extra_headers=0;
    int wav_format=0;
+   int lookahead;
+   int speex_serialno = -1;
 
    enh_enabled = 1;
 
@@ -463,7 +484,7 @@ int main(int argc, char **argv)
                        long_options, &option_index);
       if (c==-1)
          break;
-      
+
       switch(c)
       {
       case 0:
@@ -558,7 +579,7 @@ int main(int argc, char **argv)
 #endif
       fin=stdin;
    }
-   else 
+   else
    {
       fin = fopen(inFile, "rb");
       if (!fin)
@@ -572,9 +593,10 @@ int main(int argc, char **argv)
 
    /*Init Ogg data struct*/
    ogg_sync_init(&oy);
-   
+
    speex_bits_init(&bits);
    /*Main decoding loop*/
+
    while (1)
    {
       char *data;
@@ -582,7 +604,7 @@ int main(int argc, char **argv)
       /*Get the ogg buffer for writing*/
       data = ogg_sync_buffer(&oy, 200);
       /*Read bitstream from input file*/
-      nb_read = fread(data, sizeof(char), 200, fin);      
+      nb_read = fread(data, sizeof(char), 200, fin);
       ogg_sync_wrote(&oy, nb_read);
 
       /*Loop for all complete pages we got (most likely only one)*/
@@ -593,13 +615,18 @@ int main(int argc, char **argv)
             ogg_stream_init(&os, ogg_page_serialno(&og));
             stream_init = 1;
          }
+         if (ogg_page_serialno(&og) != os.serialno) {
+            /* so all streams are read. */
+            ogg_stream_reset_serialno(&os, ogg_page_serialno(&og));
+         }
          /*Add page to the bitstream*/
          ogg_stream_pagein(&os, &og);
          page_granule = ogg_page_granulepos(&og);
          page_nb_packets = ogg_page_packets(&og);
          if (page_granule>0 && frame_size)
          {
-            skip_samples = page_nb_packets*frame_size*nframes - (page_granule-last_granule);
+            /* FIXME: shift the granule values if --force-* is specified */
+            skip_samples = frame_size*(page_nb_packets*granule_frame_size*nframes - (page_granule-last_granule))/granule_frame_size;
             if (ogg_page_eos(&og))
                skip_samples = -skip_samples;
             /*else if (!ogg_page_bos(&og))
@@ -612,16 +639,22 @@ int main(int argc, char **argv)
          last_granule = page_granule;
          /*Extract all available packets*/
          packet_no=0;
-         while (!eos && ogg_stream_packetout(&os, &op)==1)
+         while (!eos && ogg_stream_packetout(&os, &op) == 1)
          {
+            if (op.bytes>=5 && !memcmp(op.packet, "Speex", 5)) {
+               speex_serialno = os.serialno;
+            }
+            if (speex_serialno == -1 || os.serialno != speex_serialno)
+               break;
             /*If first packet, process as Speex header*/
             if (packet_count==0)
             {
-               st = process_header(&op, enh_enabled, &frame_size, &rate, &nframes, forceMode, &channels, &stereo, &extra_headers, quiet);
-               if (!nframes)
-                  nframes=1;
+               st = process_header(&op, enh_enabled, &frame_size, &granule_frame_size, &rate, &nframes, forceMode, &channels, &stereo, &extra_headers, quiet);
                if (!st)
                   exit(1);
+               speex_decoder_ctl(st, SPEEX_GET_LOOKAHEAD, &lookahead);
+               if (!nframes)
+                  nframes=1;
                fout = out_file_open(outFile, rate, &channels);
 
             } else if (packet_count==1)
@@ -638,7 +671,7 @@ int main(int argc, char **argv)
                   lost=1;
 
                /*End of stream condition*/
-               if (op.e_o_s)
+               if (op.e_o_s && os.serialno == speex_serialno) /* don't care for anything except speex eos */
                   eos=1;
 
                /*Copy Ogg packet to Speex bitstream*/
@@ -671,34 +704,35 @@ int main(int argc, char **argv)
                      speex_decode_stereo_int(output, frame_size, &stereo);
 
                   if (print_bitrate) {
-                     int tmp;
+                     spx_int32_t tmp;
                      char ch=13;
                      speex_decoder_ctl(st, SPEEX_GET_BITRATE, &tmp);
                      fputc (ch, stderr);
                      fprintf (stderr, "Bitrate is use: %d bps     ", tmp);
                   }
                   /*Convert to short and save to output file*/
-		  if (strlen(outFile)!=0)
+                  if (strlen(outFile)!=0)
                   {
                      for (i=0;i<frame_size*channels;i++)
                         out[i]=le_short(output[i]);
-		  } else {
+                  } else {
                      for (i=0;i<frame_size*channels;i++)
                         out[i]=output[i];
-		  }
+                  }
                   {
                      int frame_offset = 0;
                      int new_frame_size = frame_size;
                      /*printf ("packet %d %d\n", packet_no, skip_samples);*/
+                     /*fprintf (stderr, "packet %d %d %d\n", packet_no, skip_samples, lookahead);*/
                      if (packet_no == 1 && j==0 && skip_samples > 0)
                      {
                         /*printf ("chopping first packet\n");*/
-                        new_frame_size -= skip_samples;
-                        frame_offset = skip_samples;
+                        new_frame_size -= skip_samples+lookahead;
+                        frame_offset = skip_samples+lookahead;
                      }
                      if (packet_no == page_nb_packets && skip_samples < 0)
                      {
-                        int packet_length = nframes*frame_size+skip_samples;
+                        int packet_length = nframes*frame_size+skip_samples+lookahead;
                         new_frame_size = packet_length - j*frame_size;
                         if (new_frame_size<0)
                            new_frame_size = 0;
@@ -706,15 +740,15 @@ int main(int argc, char **argv)
                            new_frame_size = frame_size;
                         /*printf ("chopping end: %d %d %d\n", new_frame_size, packet_length, packet_no);*/
                      }
-                     if (new_frame_size)
-                     {  
+                     if (new_frame_size>0)
+                     {
 #if defined WIN32 || defined _WIN32
                         if (strlen(outFile)==0)
                            WIN_Play_Samples (out+frame_offset*channels, sizeof(short) * new_frame_size*channels);
                         else
 #endif
                            fwrite(out+frame_offset*channels, sizeof(short), new_frame_size*channels, fout);
-                  
+
                         audio_size+=sizeof(short)*new_frame_size*channels;
                      }
                   }
@@ -728,7 +762,7 @@ int main(int argc, char **argv)
 
    }
 
-   if (wav_format)
+   if (fout && wav_format)
    {
       if (fseek(fout,4,SEEK_SET)==0)
       {
@@ -750,7 +784,7 @@ int main(int argc, char **argv)
 
    if (st)
       speex_decoder_destroy(st);
-   else 
+   else
    {
       fprintf (stderr, "This doesn't look like a Speex file\n");
    }
@@ -767,7 +801,7 @@ int main(int argc, char **argv)
    if (close_in)
       fclose(fin);
    if (fout != NULL)
-      fclose(fout);   
+      fclose(fout);
 
    return 0;
 }
